# -*- coding: utf-8 -*-
"""ist652_finalproj_demaria.ipynb

Automatically generated by Colaboratory.

Original file is located at
    https://colab.research.google.com/drive/1iftkMKa9xULVxh7sS_G3tigaLceKKKHe

#Import Data into workbook
"""

# IST 652 Final Project
# Mike DeMaria
# mjdemari@syr.edu

# Import libraries
import pandas as pd
from io import StringIO
from google.colab import drive

import matplotlib.pyplot as plt
import urllib.request
import json

"""# Get data sets"""

# Farmers Markets in New York State
# https://data.ny.gov/Economic-Development/Farmers-Markets-in-New-York-State/qq4h-8p86

# Retrieve data via JSON
fm_url = "https://data.ny.gov/resource/qq4h-8p86.json"
response = urllib.request.urlopen(fm_url)
fm_json_string = response.read().decode('utf-8')
fm_parsed = pd.read_json(StringIO(fm_json_string))

print("\n++++++ DATAFRAME TYPES ++++++\n")
print(fm_parsed.dtypes)
print("\n++++++ DATAFRAME SHAPE ++++++\n")
print(fm_parsed.shape)

# Agricultural Districts Acreage Totals by County
# https://data.ny.gov/Economic-Development/Agricultural-Districts-Acreage-Totals-by-County/h2id-x25a

# Retrieve data via JSON
farmland_url = "https://data.ny.gov/resource/h2id-x25a.json"
response = urllib.request.urlopen(farmland_url)
farmland_json_string = response.read().decode('utf-8')
farmland_parsed = pd.read_json(StringIO(farmland_json_string))

print("\n++++++ DATAFRAME TYPES ++++++\n")
print(farmland_parsed.dtypes)
print("\n++++++ DATAFRAME SHAPE ++++++\n")
print(farmland_parsed.shape)

# Average Income and Tax Liability of Full-Year Residents by County
# https://data.ny.gov/Government-Finance/Average-Income-and-Tax-Liability-of-Full-Year-Resi/2w9v-ejxd

# Retrieve data via JSON
income_url = "https://data.ny.gov/resource/2w9v-ejxd.json?$limit=1500"
response = urllib.request.urlopen(income_url)
income_json_string = response.read().decode('utf-8')
income_parsed = pd.read_json(StringIO(income_json_string))

print("\n++++++ DATAFRAME TYPES ++++++\n")
print(income_parsed.dtypes)
print("\n++++++ DATAFRAME SHAPE ++++++\n")
print(income_parsed.shape)

# Population Density
# https://www.health.ny.gov/statistics/vital_statistics/2018/table02.htm

# I converted the table on the page into a csv, as they do not offer an API.
drive.mount('/content/drive')

datapath = '/content/drive/MyDrive/IST652_Fall22_MJD/ny_popdensity.csv'
pop_density_parsed = pd.read_csv(datapath, sep=',')


print("\n++++++ DATAFRAME TYPES ++++++\n")
print(pop_density_parsed.dtypes)
print("\n++++++ DATAFRAME SHAPE ++++++\n")
print(pop_density_parsed.shape)

"""#Clean and convert data"""

# fm - Farmers Market dataframe
# The fields were generally parsed correctly.  Let's explicitly convert the object columns to strings.
fm_parsed["county"] = fm_parsed["county"].astype('string')
fm_parsed["county"] = fm_parsed["county"].str.lower()
fm_parsed["city"] = fm_parsed["city"].astype('string')
fm_parsed["state"] = fm_parsed["state"].astype('string')
fm_parsed["state"] = fm_parsed["state"].str.upper()
fm_parsed["zip"] = fm_parsed["zip"].astype('string')
fm_parsed["snap_status"] = fm_parsed["snap_status"].astype('string')
fm_parsed["snap_status"] = fm_parsed["snap_status"].str.upper()


# Eliminate nulls and any locations outside of NYS
fm_cleaned = fm_parsed[(fm_parsed["county"] != "") & 
                         (fm_parsed["city"] != "") &
                         (fm_parsed["state"] == "NY") &
                         (fm_parsed["zip"] != "") &
                         ((fm_parsed["snap_status"] == "Y") | (fm_parsed["snap_status"] == "N")) &
                         (fm_parsed["latitude"].notnull()) &
                         (fm_parsed["longitude"].notnull())
                         ]
fm = fm_cleaned[["county","city","state","zip","snap_status","latitude","longitude"]]
# We can see from this that the data is surprisingly clean.  All fields were filled in.
print(fm.shape)
print(fm.dtypes)
print(fm.head(5))

# farmland - Arable land dataframe
# The fields were generally parsed correctly.  Let's explicitly convert the object columns to strings.
farmland_parsed["county"] = farmland_parsed["county"].astype('string')
farmland_parsed["county"] = farmland_parsed["county"].str.lower()

# Eliminate nulls
farmland_cleaned = farmland_parsed[(farmland_parsed["county"] != "") & 
                         (farmland_parsed["total_acres"].notnull()) &
                         (farmland_parsed["farmed_acres"].notnull())
                         ]

# Total land that can be farms, and total land that are actually farms, are different.
# Total land will be greater than farmed land.  Since we're only focused on active farms
# growing and selling products, we'll only look at farmed acres.
farmland = farmland_cleaned[["county","farmed_acres"]]

# We can see from this that the data is surprisingly clean.  All fields were filled in.
print(farmland.shape)
print(farmland.dtypes)
print(farmland.head(5))
farmland.describe

# income - Average income dataframe
# The fields were generally parsed correctly.  Let's explicitly convert the object columns to strings.
income_parsed["place_of_residence"] = income_parsed["place_of_residence"].astype('string')
income_parsed["place_of_residence"] = income_parsed["place_of_residence"].str.lower()

# Eliminate nulls.  As multiple years are included in this dataset, we'll scope down to the most recent year available.
income_cleaned = income_parsed[(income_parsed["place_of_residence"] != "") & 
                               (income_parsed["tax_year"] == 2014) &
                         (income_parsed["average_ny_agi_of_all_returns"].notnull())
                         ]

# Taxable returns just includes returns that pay income taxes.  Low income earners will not fall into this category.
# The 'all returns' column reflects a lower adjusted gross income than the 'taxable returns'.  As we're trying to find
# a correlation between income and farmers market availability, we'll go with the all return option.
income = income_cleaned[["place_of_residence","average_ny_agi_of_all_returns"]]
income = income.rename(columns={"place_of_residence" : "county", "average_ny_agi_of_all_returns" : "agi"})
# We can see from this that the data is surprisingly clean.  All fields were filled in.
print(income.shape)
print(income.dtypes)
print(income.head(5))

# density - Population density dataframe

# Convert the relevant fields into proper strings or numbers.
pop_density_parsed["County"] = pop_density_parsed["County"].astype('string')
pop_density_parsed["County"] = pop_density_parsed["County"].str.lower()
pop_density_parsed["2018 Population Estimate"] = pop_density_parsed["2018 Population Estimate"].str.replace(',','').astype('float')
pop_density_parsed["2010 Land Area Square Miles"] = pop_density_parsed["2010 Land Area Square Miles"].str.replace(',','').astype('float')

# Eliminate nulls.  As multiple years are included in this dataset, we'll scope down to the most recent year available.
pop_density_cleaned = pop_density_parsed[(pop_density_parsed["County"] != "") & 
                         (pop_density_parsed["2018 Population Estimate"].notnull()) &
                         (pop_density_parsed["2010 Land Area Square Miles"].notnull())
                         ]

# Taxable returns just includes returns that pay income taxes.  Low income earners will not fall into this category.
# The 'all returns' column reflects a lower adjusted gross income than the 'taxable returns'.  As we're trying to find
# a correlation between income and farmer market availability, we'll go with the all return option.
pop_density = pop_density_cleaned[["County","2018 Population Estimate", "2010 Land Area Square Miles"]]

pop_density = pop_density.rename(columns={"County" : "county", "2018 Population Estimate" : "pop", "2010 Land Area Square Miles" : "area"})

# Farms are in acres, this land mass is in miles.  To convert, 1 square mile = 640 acres.
# While we're at it, let's compute the population density in people per acre.
# It's less common of a unit than per mile density, but the math works out the same.
pop_density["area"] = pop_density["area"] * 640

pop_density["density"] = pop_density["pop"] / pop_density["area"]

# We can see from this that the data is surprisingly clean.  All fields were filled in.
print(pop_density.shape)
print(pop_density.dtypes)
print(pop_density.head(5))

"""#Create a  dataframe that merges the counties, markets, farmland, pop and income"""

# First, we'll create a dataframe containing all the counties.
# Then, we'll find the following for each county: population, density, farm acres, area, average income
# Then, we'll add up the farmers markets
# Then, we'll compute the derived columns

pop_density = pop_density.set_index('county')
farmland = farmland.set_index('county')
income = income.set_index('county')

fm_joined = pd.DataFrame(columns=['county','income','farmland','total area','pop','density','farmer markets','fm with snap', 'fm without snap','percentage with snap', 'percentage as farmland']);

for county in fm["county"].unique():
  temp = pd.DataFrame({"county" : [county]})
  temp["county"] = county
  fm_joined = fm_joined.append(temp, ignore_index=True)

fm_joined = fm_joined.set_index('county')

# Density
for county in fm_joined.index:
  area = 0
  pop = 0
  density = 0
  try:
    searchitem = pop_density.loc[[county]]
    area = searchitem['area'].values
    pop = searchitem['pop'].values
    density = searchitem['density'].values
  except:
    area = 0
    pop = 0
    density = 0

  fm_joined.at[county,'total area'] = area
  fm_joined.at[county,'pop'] = pop
  fm_joined.at[county,'density'] = density


# Farmland
for county in fm_joined.index:
  searchvalue = 0
  try:
    searchitem = farmland.loc[[county]]
    searchvalue = searchitem['farmed_acres'].values
  except:
    searchvalue = 0

  fm_joined.at[county,'farmland'] = searchvalue
                        
# Income
for county in fm_joined.index:
  searchvalue = 0
  try:
    searchitem = income.loc[[county]]
    searchvalue = searchitem['agi'].values
  except:
    searchvalue = 0

  fm_joined.at[county,'income'] = searchvalue

# Farmers Markets
fm_nosnap = fm[fm["snap_status"] == "N"]
fm_snap = fm[fm["snap_status"] == "Y"]

agg_nosnap = pd.DataFrame(fm_nosnap.groupby(["county"])['county'].count())
agg_snap = pd.DataFrame(fm_snap.groupby(["county"])['county'].count())

# With Snap
for county in fm_joined.index:
  snap = 0
  
  try:
    searchitem = agg_snap.loc[[county]]
    searchvalue = searchitem['county'].values
  except:
    searchvalue = 0

  fm_joined.at[county,'fm with snap'] = searchvalue

# No Snap
for county in fm_joined.index:
  nosnap = 0
  
  try:
    searchitem = agg_nosnap.loc[[county]]
    searchvalue = searchitem['county'].values
  except:
    searchvalue = 0

  fm_joined.at[county,'fm without snap'] = searchvalue

# Clean up the columns by converting to the correct data types

fm_joined["income"] = fm_joined["income"].astype('float64')
fm_joined["farmland"] = fm_joined["farmland"].astype('float64')
fm_joined["total area"] = fm_joined["total area"].astype('float64')
fm_joined["pop"] = fm_joined["pop"].astype('float64')
fm_joined["density"] = fm_joined["density"].astype('float64')
fm_joined["fm with snap"] = fm_joined["fm with snap"].astype('float64')
fm_joined["fm without snap"] = fm_joined["fm without snap"].astype('float64')

# Compute derived columns
# Fortunately, Python handles the divide by zero possibility and sets the value as infinite without throwing an error.
fm_joined['farmer markets'] = fm_joined['fm with snap'] + fm_joined['fm without snap']
fm_joined['percentage with snap'] = (fm_joined['fm with snap'] / fm_joined['farmer markets'] ) * 100
fm_joined['percentage as farmland'] = (fm_joined['farmland'] / fm_joined['total area'] ) * 100

# Cast columns
fm_joined["farmer markets"] = fm_joined["farmer markets"].astype('float64')
fm_joined["percentage with snap"] = fm_joined["percentage with snap"].astype('float64')
fm_joined["percentage as farmland"] = fm_joined["percentage as farmland"].astype('float64')

print("\n++++++ DATAFRAME TYPES ++++++\n")
print(fm_joined.dtypes)
print("\n++++++ DATAFRAME SHAPE ++++++\n")
print(fm_joined.shape)
print(fm_joined.head(5))

"""# Plots"""

# Exploratory plots.
# Starting with markets per income

markets_by_income = fm_joined.groupby(["income"])["farmer markets"].mean()
markets_by_income.plot(kind="bar", title="Markets by Income")

# Bucket into income ranges
# These buckets are based on NY's groupings as illustrated at
# https://data.ny.gov/Government-Finance/Income-Tax-Components-of-Full-Year-Residents-by-Si/5kgr-h5g5


incomeranges = {"Under 30k": 0,
                "30k-40k":0,
                "40k-50k":0,
                "50k-60k":0,
                "60k-75k":0,
                "75k-100k":0,
                "Over 100k":0,                
                }

for county in fm_joined.index:
  loc = pd.DataFrame(fm_joined.loc[[county]])
  if(loc['income'].values < 30000):
    incomeranges["Under 30k"] = incomeranges.get("Under 30k") + loc['farmer markets'].values[0]
    continue

  if((loc['income'].values >= 30000) & (loc['income'].values < 40000)):
    incomeranges["30k-40k"] = incomeranges.get("30k-40k") + loc['farmer markets'].values[0]
    continue

  if((loc['income'].values >= 40000) & (loc['income'].values < 50000)):
    incomeranges["40k-50k"] = incomeranges.get("40k-50k") + loc['farmer markets'].values[0]
    continue

  if((loc['income'].values >= 50000) & (loc['income'].values < 60000)):
    incomeranges["50k-60k"] = incomeranges.get("50k-60k") + loc['farmer markets'].values[0]
    continue

  if((loc['income'].values >= 60000) & (loc['income'].values < 75000)):
    incomeranges["60k-75k"] = incomeranges.get("60k-75k") + loc['farmer markets'].values[0]
    continue

  if((loc['income'].values >= 75000) & (loc['income'].values < 100000)):
    incomeranges["75k-100k"] = incomeranges.get("75k-100k") + loc['farmer markets'].values[0]
    continue

  if(loc['income'].values >= 100000):
    incomeranges["Over 100k"] = incomeranges.get("Over 100k") + loc['farmer markets'].values[0]
    continue

plt.figure(figsize=(12, 4), dpi=80)
plt.title("Markets per income bucket")
plt.bar(range(len(incomeranges)), list(incomeranges.values()), tick_label=list(incomeranges.keys()))

# Exploratory plots.
# Markets per farmland percentage

markets_by_farmland = fm_joined.groupby(["percentage as farmland"])["farmer markets"].mean()
markets_by_farmland.plot(kind="bar", title="Markets by Farmland Percentage")

# Market per farm percentage


farmranges = {"Under 5%": 0,
                "5%-10%": 0,
                "10%-15%": 0,
                "15%-20%": 0,
                "20%-25%": 0,              
                "25%-30%":0,
                "30%-35%":0,
                "35%-40%":0,
                "40%-45%":0,
                "45%-50%":0,
                "Over 50%":0                
                }

for county in fm_joined.index:
  loc = pd.DataFrame(fm_joined.loc[[county]])
  if(loc['percentage as farmland'].values < 5):
    farmranges["Under 5%"] = farmranges.get("Under 5%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 5) & (loc['percentage as farmland'].values < 10)):
    farmranges["5%-10%"] = farmranges.get("5%-10%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 10) & (loc['percentage as farmland'].values < 15)):
    farmranges["10%-15%"] = farmranges.get("10%-15%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 15) & (loc['percentage as farmland'].values < 20)):
    farmranges["15%-20%"] = farmranges.get("15%-20%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 20) & (loc['percentage as farmland'].values < 25)):
    farmranges["20%-25%"] = farmranges.get("20%-25%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 25) & (loc['percentage as farmland'].values < 30)):
    farmranges["25%-30%"] = farmranges.get("25%-30%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 30) & (loc['percentage as farmland'].values < 35)):
    farmranges["30%-35%"] = farmranges.get("30%-35%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 35) & (loc['percentage as farmland'].values < 40)):
    farmranges["35%-40%"] = farmranges.get("35%-40%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 40) & (loc['percentage as farmland'].values < 45)):
    farmranges["40%-45%"] = farmranges.get("40%-45%") + loc['farmer markets'].values[0]
    continue

  if((loc['percentage as farmland'].values >= 45) & (loc['percentage as farmland'].values < 50)):
    farmranges["45%-50%"] = farmranges.get("45%-50%") + loc['farmer markets'].values[0]
    continue

  # The 100 makes sure those infinite divide by zero ones do not get counted
  if((loc['percentage as farmland'].values >= 50) & (loc['percentage as farmland'].values <= 100)):
    farmranges["Over 50%"] = farmranges.get("Over 50%") + loc['farmer markets'].values[0]
    continue

plt.figure(figsize=(12, 4), dpi=80)
plt.title("Markets by farmland percentage")
plt.bar(range(len(farmranges)), list(farmranges.values()), tick_label=list(farmranges.keys()))

# Exploratory plots.
# Markets per population density

markets_by_popdensity = fm_joined.groupby(["density"])["farmer markets"].mean()
markets_by_popdensity.plot(kind="bar", title="Markets by Population Density")

# Market per population density


popranges = {"Under 0.1": 0,
                "0.1-0.2": 0,
                "0.2-0.3": 0,
                "0.3-0.4": 0,
                "0.4-0.5": 0,              
                "0.5-1.0":0,
                "1.0-5.0":0,
                "5.0-25.0":0,
                "Over 25.0":0                
                }

for county in fm_joined.index:
  loc = pd.DataFrame(fm_joined.loc[[county]])
  if(loc['density'].values < 0.1):
    popranges["Under 0.1"] = popranges.get("Under 0.1") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 0.1) & (loc['density'].values < 0.2)):
    popranges["0.1-0.2"] = popranges.get("0.1-0.2") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 0.2) & (loc['density'].values < 0.3)):
    popranges["0.2-0.3"] = popranges.get("0.2-0.3") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 0.3) & (loc['density'].values < 0.4)):
    popranges["0.3-0.4"] = popranges.get("0.3-0.4") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 0.4) & (loc['density'].values < 0.5)):
    popranges["0.4-0.5"] = popranges.get("0.4-0.5") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 0.5) & (loc['density'].values < 1.0)):
    popranges["0.5-1.0"] = popranges.get("0.5-1.0") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 1.0) & (loc['density'].values < 5.0)):
    popranges["1.0-5.0"] = popranges.get("1.0-5.0") + loc['farmer markets'].values[0]
    continue

  if((loc['density'].values >= 5.0) & (loc['density'].values < 25.0)):
    popranges["5.0-25.0"] = popranges.get("5.0-25.0") + loc['farmer markets'].values[0]
    continue

  # The 9999 makes sure those infinite divide by zero ones do not get counted
  if((loc['density'].values >= 25.0) & (loc['density'].values < 9999.0)):
    popranges["Over 25.0"] = popranges.get("Over 25.0") + loc['farmer markets'].values[0]
    continue

plt.figure(figsize=(12, 4), dpi=80)
plt.title("Markets by pop density")
plt.bar(range(len(popranges)), list(popranges.values()), tick_label=list(popranges.keys()))

"""# Advanced Projects Component - Seaborn"""

!pip install seaborn
import seaborn as sns

sns.regplot(x="income", y="farmer markets", data=fm_joined)

sns.regplot(x="density", y="farmer markets", data=fm_joined)

sns.regplot(x="percentage as farmland", y="farmer markets", data=fm_joined)

sns.lmplot(x="income", y="farmer markets", hue="percentage as farmland", data=fm_joined)

sns.jointplot(x="income", y="farmer markets", data=fm_joined, kind="reg")

sns.jointplot(x="percentage as farmland", y="farmer markets", data=fm_joined, kind="reg")

sns.jointplot(x="density", y="farmer markets", data=fm_joined, kind="reg")

sns.pairplot(fm_joined, x_vars=["income","density","percentage as farmland"], y_vars=["farmer markets"], kind="reg")

# Let's do one last one, plot by farmland and colored by density
# However, we will need to bucket density, otherwise it's too many colors.

for county in fm_joined.index:
  loc = pd.DataFrame(fm_joined.loc[[county]])
  if(loc['density'].values < 0.2):
    fm_joined.at[county,'Density Bucket'] = "Very Small"
    continue

  if((loc['density'].values >= 0.2) & (loc['density'].values < 2)):
    fm_joined.at[county,'Density Bucket'] = "Small"
    continue

  if((loc['density'].values >= 2) & (loc['density'].values < 50)):
    fm_joined.at[county,'Density Bucket'] = "Medium"
    continue

  # The 9999 makes sure those infinite divide by zero ones do not get counted
  if((loc['density'].values >= 50.0) & (loc['density'].values < 9999.0)):
    fm_joined.at[county,'Density Bucket'] = "Large"
    continue


sns.lmplot(x="percentage as farmland", y="farmer markets", data=fm_joined, hue="Density Bucket")